/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "messagefilter.h"

#include <qvariant.h>

using namespace IRC;

MessageFilter::MessageFilter( QObject *parent, const char *name )
    : QObject( parent, name )
{
}

MessageFilter::~MessageFilter()
{
}

MessageFilterGroup::MessageFilterGroup( QObject *parent, const char *name )
    : MessageFilter( parent, name )
{
}

MessageFilterGroup::~MessageFilterGroup()
{
    for ( MessageFilterListIterator it( m_filters );
          it.current(); ++it )
        disconnect( it.current(), SIGNAL( destroyed( QObject * ) ),
                    this, SLOT( filterDestroyed( QObject * ) ) );
}

MessageFilter::Result MessageFilterGroup::filterMessage( Message &msg, Window *window )
{
    MessageFilterListIterator it( m_filters );
    for ( ; it.current(); ++it ) {
        MessageFilter::Result result = it.current()->filterMessage( msg, window );

        if ( result != MessageFilter::ContinueFiltering )
            return result;
    }

    return ContinueFiltering;
}

bool MessageFilterGroup::addFilter( MessageFilter *filter )
{
    if ( m_filters.findRef( filter ) != -1 )
        return false;

    m_filters.append( filter );

    connect( filter, SIGNAL( destroyed( QObject * ) ),
             this, SLOT( filterDestroyed( QObject * ) ) );

    return true;
}

bool MessageFilterGroup::removeFilter( MessageFilter *filter )
{
    if ( m_filters.findRef( filter ) == -1 )
        return false;

    disconnect( filter, SIGNAL( destroyed( QObject * ) ),
                this, SLOT( filterDestroyed( QObject * ) ) );

    return m_filters.removeRef( filter );
}

void MessageFilterGroup::filterDestroyed( QObject *object )
{
    MessageFilter *filter = static_cast<MessageFilter *>( object );
    removeFilter( filter );
}

NamedMessageFilter::NamedMessageFilter( const QString &messageType, QObject *parent, const char *name )
    : MessageFilter( parent, name ), m_messageType( messageType )
{
}

QString NamedMessageFilter::messageType() const
{
    return m_messageType;
}

NamedMessageFilterGroup::NamedMessageFilterGroup( QObject *parent, const char *name )
    : MessageFilter( parent, name )
{
}

NamedMessageFilterGroup::~NamedMessageFilterGroup()
{
    for ( MessageFilterMapIterator it( m_filters );
          it.current(); ++it )
        disconnect( it.current(), SIGNAL( destroyed( QObject * ) ),
                    this, SLOT( filterDestroyed( QObject * ) ) );
}

MessageFilter::Result NamedMessageFilterGroup::filterMessage( Message &msg, Window *window )
{
    QString name = msg.type();

    MessageFilter *filter = m_filters[ name ];
    if ( !filter )
        return ContinueFiltering;

    return filter->filterMessage( msg, window );
}

bool NamedMessageFilterGroup::addFilter( const QString &name, MessageFilter *filter )
{
    if ( m_filters[ name ] != 0 )
        return false;

    m_filters.insert( name, filter );

    connect( filter, SIGNAL( destroyed( QObject * ) ),
             this, SLOT( filterDestroyed( QObject * ) ) );

    return true;
}

bool NamedMessageFilterGroup::addFilter( MessageFilter *filter )
{
    QVariant messageTypeProp = filter->property( "messageType" );
    if ( !messageTypeProp.isValid() ) 
        return false;

    return addFilter( messageTypeProp.toString(), filter );
}

bool NamedMessageFilterGroup::removeFilter( const QString &name )
{
    MessageFilter *filter = m_filters[ name ];

    if ( !filter )
        return false;

    disconnect( filter, SIGNAL( destroyed( QObject * ) ),
                this, SLOT( filterDestroyed( QObject * ) ) );

    m_filters.remove( name );
    return true;
}

bool NamedMessageFilterGroup::removeFilter( MessageFilter *filter )
{
    QVariant messageTypeProp = filter->property( "messageType" );
    if ( !messageTypeProp.isValid() )
        return false;

    return removeFilter( messageTypeProp.toString() );
}

void NamedMessageFilterGroup::filterDestroyed( QObject *object )
{
    MessageFilter *filter = static_cast<MessageFilter *>( object );

    MessageFilterMapIterator it( m_filters );
    while ( it.current() )
        if ( it.current() == filter )
            m_filters.remove( it.currentKey() );
        else
            ++it;
}

#include "messagefilter.moc"

/* vim: et sw=4 ts=4
 */
